USE AdventureWorks
GO

IF OBJECT_ID ('dbo.BrokerConversation','U') IS NOT NULL
   DROP TABLE dbo.BrokerConversation
GO
CREATE TABLE [dbo].[BrokerConversation]
(
[ch] [uniqueidentifier] NULL,
[service_name] [nvarchar] (512) COLLATE SQL_Latin1_General_CP1_CI_AS NULL
) ON [PRIMARY]
GO


IF OBJECT_ID ('dbo.StartSyncConversation','P') IS NOT NULL
   DROP PROCEDURE dbo.StartSyncConversation
GO
CREATE PROC [dbo].[StartSyncConversation]
AS
SET NOCOUNT ON;

    DECLARE @ch UNIQUEIDENTIFIER;
    DECLARE @service_name nvarchar(512) = N'//AsyncSync/SyncService';
    
    SELECT @ch=[ch]
    FROM [dbo].[BrokerConversation]
    WHERE [service_name] = @service_name;
    
    IF @ch IS NOT NULL
        BEGIN
        DELETE FROM [dbo].[BrokerConversation]
        WHERE [service_name] = @service_name;
        
        END CONVERSATION @ch;
        END

    BEGIN DIALOG @ch
       FROM SERVICE [//AsyncSync/SyncService]
       TO SERVICE N'//AsyncSync/SyncProcessSvc'
       ON CONTRACT [//AsyncSync/SyncContract]
       WITH
           ENCRYPTION = ON;

    INSERT INTO [dbo].[BrokerConversation]
        (ch, service_name)
    VALUES (@ch, @service_name);

GO


IF OBJECT_ID ('Sales.trSyncCustomer','TR') IS NOT NULL
   DROP TRIGGER Sales.trSyncCustomer
GO

CREATE TRIGGER Sales.trSyncCustomer
   ON  [Sales].[Customer]
   FOR INSERT, UPDATE, DELETE
AS 

  DECLARE @InitDlgHandle UNIQUEIDENTIFIER;
  DECLARE @ChangeMsg XML;
  DECLARE @ChangeCnt int;
    DECLARE @service_name nvarchar(512) = N'//AsyncSync/SyncService';
    
    SELECT @InitDlgHandle=[ch]
    FROM [dbo].[BrokerConversation]
    WHERE [service_name] = @service_name;
  
  BEGIN TRY
      
      set @ChangeMsg = (
        SELECT c.[AccountNumber]
          FROM [Sales].[Customer] c
        INNER JOIN inserted i on c.[AccountNumber] = i.[AccountNumber]
        FOR XML RAW, ELEMENTS, ROOT ('Customer')
        );
      
      BEGIN TRANSACTION;

      SEND ON CONVERSATION @InitDlgHandle
           MESSAGE TYPE [//AsyncSync/Customer]
           (@ChangeMsg);

      COMMIT TRANSACTION;

  END TRY
  BEGIN CATCH

    INSERT INTO dbo.ErrorLog (ErrorTime, UserName, ErrorNumber, ErrorSeverity,
        ErrorState, ErrorProcedure, ErrorLine, ErrorMessage)
    VALUES (getdate(), USER_NAME(), ERROR_NUMBER(), ERROR_SEVERITY(),
        ERROR_STATE(), ERROR_PROCEDURE(), ERROR_LINE(), ERROR_MESSAGE())
  
  END CATCH
  
GO


IF OBJECT_ID ('Production.trSyncProduct','TR') IS NOT NULL
   DROP TRIGGER Production.trSyncProduct
GO

CREATE TRIGGER Production.trSyncProduct
   ON  [Production].[Product]
   FOR INSERT, UPDATE, DELETE
AS 

  DECLARE @InitDlgHandle UNIQUEIDENTIFIER;
  DECLARE @ChangeMsg XML;
  DECLARE @ChangeCnt int;
    DECLARE @service_name nvarchar(512) = N'//AsyncSync/SyncService';
    
    SELECT @InitDlgHandle=[ch]
    FROM [dbo].[BrokerConversation]
    WHERE [service_name] = @service_name;
  
  BEGIN TRY
      
      set @ChangeMsg = (
        SELECT c.[ProductNumber]
          FROM [Production].[Product] c
        INNER JOIN inserted i on c.[ProductNumber] = i.[ProductNumber]
        FOR XML RAW, ELEMENTS, ROOT ('Product')
        );
      
      BEGIN TRANSACTION;

      SEND ON CONVERSATION @InitDlgHandle
           MESSAGE TYPE [//AsyncSync/Product]
           (@ChangeMsg);

      COMMIT TRANSACTION;

  END TRY
  BEGIN CATCH

    INSERT INTO dbo.ErrorLog (ErrorTime, UserName, ErrorNumber, ErrorSeverity,
        ErrorState, ErrorProcedure, ErrorLine, ErrorMessage)
    VALUES (getdate(), USER_NAME(), ERROR_NUMBER(), ERROR_SEVERITY(),
        ERROR_STATE(), ERROR_PROCEDURE(), ERROR_LINE(), ERROR_MESSAGE())
  
  END CATCH
  
GO

IF OBJECT_ID ('Person.trSyncContact','TR') IS NOT NULL
   DROP TRIGGER Person.trSyncContact
GO

CREATE TRIGGER Person.trSyncContact
   ON  [Person].[Contact]
   FOR INSERT, UPDATE, DELETE
AS 

  DECLARE @InitDlgHandle UNIQUEIDENTIFIER;
  DECLARE @ChangeMsg XML;
  DECLARE @ChangeCnt int;
    DECLARE @service_name nvarchar(512) = N'//AsyncSync/SyncService';
    
    SELECT @InitDlgHandle=[ch]
    FROM [dbo].[BrokerConversation]
    WHERE [service_name] = @service_name;
  
  BEGIN TRY
      
      set @ChangeMsg = (
        SELECT SUBSTRING(c.[EmailAddress],1,(CHARINDEX('@',c.[EmailAddress],1) - 1)) AS UserID
	      ,c.[FirstName]
	      ,c.[LastName]
	      ,c.[EmailAddress]
	      ,c.[Phone]
	    FROM [Person].[Contact] c
        INNER JOIN inserted i on c.[ContactID] = i.[ContactID]
        FOR XML RAW, ELEMENTS, ROOT ('Contact')
        );
      
      BEGIN TRANSACTION;

      SEND ON CONVERSATION @InitDlgHandle
           MESSAGE TYPE [//AsyncSync/Contact]
           (@ChangeMsg);

      COMMIT TRANSACTION;

  END TRY
  BEGIN CATCH

    INSERT INTO dbo.ErrorLog (ErrorTime, UserName, ErrorNumber, ErrorSeverity,
        ErrorState, ErrorProcedure, ErrorLine, ErrorMessage)
    VALUES (getdate(), USER_NAME(), ERROR_NUMBER(), ERROR_SEVERITY(),
        ERROR_STATE(), ERROR_PROCEDURE(), ERROR_LINE(), ERROR_MESSAGE())
  
  END CATCH
  
GO

