USE AdventureWorks
GO

IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[BrokerMessages]') AND type = N'U')
DROP TABLE [dbo].[BrokerMessages]
GO

CREATE TABLE  [dbo].[BrokerMessages] (
	id		INT IDENTITY(1,1) NOT NULL,
	ch		UNIQUEIDENTIFIER,
	messagetypename	NVARCHAR(256),
	messagebody	XML
	)
GO

IF OBJECT_ID ('dbo.BrokerConversation','U') IS NOT NULL
   DROP TABLE dbo.BrokerConversation
GO
CREATE TABLE [dbo].[BrokerConversation]
(
	[ch] [uniqueidentifier] NULL,
	[service_name] [nvarchar] (512) COLLATE SQL_Latin1_General_CP1_CI_AS NULL
)
GO
IF OBJECT_ID ('dbo.[ErrorLog]','U') IS NOT NULL
   DROP TABLE dbo.[ErrorLog]
GO
CREATE TABLE [dbo].[ErrorLog](
	[ErrorLogID] [int] IDENTITY(1,1) NOT NULL,
	[ErrorTime] [datetime] NOT NULL,
	[UserName] [sysname] NOT NULL,
	[ErrorNumber] [int] NOT NULL,
	[ErrorSeverity] [int] NULL,
	[ErrorState] [int] NULL,
	[ErrorProcedure] [nvarchar](126) NULL,
	[ErrorLine] [int] NULL,
	[ErrorMessage] [nvarchar](4000) NOT NULL,
	[messagebody] [XML] NULL,
 CONSTRAINT [PK_ErrorLog_ErrorLogID] PRIMARY KEY CLUSTERED 
(
	[ErrorLogID] ASC
)
) ON [PRIMARY]

GO
IF OBJECT_ID ('dbo.StartSyncConversation','P') IS NOT NULL
   DROP PROCEDURE dbo.StartSyncConversation
GO
CREATE PROC [dbo].[StartSyncConversation]
AS
SET NOCOUNT ON;

    DECLARE @ch UNIQUEIDENTIFIER;
    DECLARE @service_name nvarchar(512) = N'//AsyncSync/SyncService';
    
    SELECT @ch=[ch]
    FROM [dbo].[BrokerConversation]
    WHERE [service_name] = @service_name;
    
    IF @ch IS NOT NULL
        BEGIN
        DELETE FROM [dbo].[BrokerConversation]
        WHERE [service_name] = @service_name;
        
        END CONVERSATION @ch;
        END

    BEGIN DIALOG @ch
       FROM SERVICE [//AsyncSync/SyncService]
       TO SERVICE N'//AsyncSync/SyncProcessSvc'
       ON CONTRACT [//AsyncSync/SyncContract]
       WITH
           ENCRYPTION = ON;

    INSERT INTO [dbo].[BrokerConversation]
        (ch, service_name)
    VALUES (@ch, @service_name);

GO

IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[ProcessSyncMessages]') AND type in (N'P', N'PC'))
DROP PROCEDURE [dbo].[ProcessSyncMessages]
GO

CREATE PROCEDURE [dbo].[ProcessSyncMessages]
AS
SET NOCOUNT ON
	DECLARE @ch UNIQUEIDENTIFIER
	DECLARE @messagetypename NVARCHAR(256)
	DECLARE	@messagebody XML
	DECLARE @responsemessage XML;

	WHILE (1=1)
	BEGIN
		BEGIN TRY
			BEGIN TRANSACTION

			WAITFOR (
				RECEIVE TOP(1)
					@ch = conversation_handle,
					@messagetypename = message_type_name,
					@messagebody = CAST(message_body AS XML)
				FROM SyncProcessQueue
			), TIMEOUT 60000

			IF (@@ROWCOUNT = 0)
			BEGIN
				ROLLBACK TRANSACTION
				BREAK
			END
			
			INSERT INTO [dbo].[BrokerMessages] (ch, messagetypename, messagebody)
			VALUES (@ch, @messagetypename, @messagebody);

			-- End the conversation
			--END CONVERSATION @ch;

			COMMIT TRANSACTION
		END TRY
		BEGIN CATCH
				
			ROLLBACK TRANSACTION

			INSERT INTO dbo.ErrorLog (ErrorTime, UserName, ErrorNumber, ErrorSeverity,
				ErrorState, ErrorProcedure, ErrorLine, ErrorMessage, MessageBody)
			VALUES (getdate(), USER_NAME(), ERROR_NUMBER(), ERROR_SEVERITY(),
				ERROR_STATE(), ERROR_PROCEDURE(), ERROR_LINE(), ERROR_MESSAGE(), @messagebody)

			--END CONVERSATION @ch;
			
			INSERT INTO [dbo].[BrokerMessages] (ch, messagetypename, messagebody)
			VALUES (@ch, @messagetypename, @messagebody);

		END CATCH
	END
GO

IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[ProcessProduction_Product]') AND type in (N'P', N'PC'))
DROP PROCEDURE [dbo].[ProcessProduction_Product]
GO

CREATE PROCEDURE [dbo].[ProcessProduction_Product]
	@messagebody XML
WITH EXECUTE AS 'SyncUser'
AS
  SET NOCOUNT ON
  BEGIN TRY
  
    -- Create a temporary table to hold the message results
    CREATE TABLE #Production_Product(
    	[Action] [nchar](1) NOT NULL,
    	[ProductID] [int] NOT NULL,
	[Name] [nvarchar](50) NULL,
	[ProductNumber] [nvarchar](25) NULL,
	[MakeFlag] [bit] NULL,
	[FinishedGoodsFlag] [bit] NULL,
	[Color] [nvarchar](15) NULL,
	[SafetyStockLevel] [smallint] NULL,
	[ReorderPoint] [smallint] NULL,
	[StandardCost] [money] NULL,
	[ListPrice] [money] NULL,
	[Size] [nvarchar](5) NULL,
	[SizeUnitMeasureCode] [nchar](3) NULL,
	[WeightUnitMeasureCode] [nchar](3) NULL,
	[Weight] [decimal](8, 2) NULL,
	[DaysToManufacture] [int] NULL,
	[ProductLine] [nchar](2) NULL,
	[Class] [nchar](2) NULL,
	[Style] [nchar](2) NULL,
	[ProductSubcategoryID] [int] NULL,
	[ProductModelID] [int] NULL,
	[SellStartDate] [datetime] NULL,
	[SellEndDate] [datetime] NULL,
	[DiscontinuedDate] [datetime] NULL
    );
    
    -- Use XQuery to shred the message and insert it into the temporary table
    INSERT INTO #Production_Product
    select  a.value(N'(./Action)[1]', N'nchar(1)') as [Action],
        a.value(N'(./ProductID)[1]', N'int') as [ProductID],
        a.value(N'(./Name)[1]', N'nvarchar(50)') as [Name],
        a.value(N'(./ProductNumber)[1]', N'nvarchar(25)') as [ProductNumber],
        a.value(N'(./MakeFlag)[1]', N'bit') as [MakeFlag],
        a.value(N'(./FinishedGoodsFlag)[1]', N'bit') as [FinishedGoodsFlag],
        a.value(N'(./Color)[1]', N'nvarchar(15)') as [Color],
        a.value(N'(./SafetyStockLevel)[1]', N'smallint') as [SafetyStockLevel],
        a.value(N'(./ReorderPoint)[1]', N'smallint') as [ReorderPoint],
        a.value(N'(./StandardCost)[1]', N'money') as [StandardCost],
        a.value(N'(./ListPrice)[1]', N'money') as [ListPrice],
        a.value(N'(./Size)[1]', N'nvarchar(5)') as [Size],
        a.value(N'(./SizeUnitMeasureCode)[1]', N'nchar(3)') as [SizeUnitMeasureCode],
        a.value(N'(./WeightUnitMeasureCode)[1]', N'nchar(3)') as [WeightUnitMeasureCode],
        a.value(N'(./Weight)[1]', N'decimal(8, 2)') as [Weight],
        a.value(N'(./DaysToManufacture)[1]', N'int') as [DaysToManufacture],
        a.value(N'(./ProductLine)[1]', N'nchar(2)') as [ProductLine],
        a.value(N'(./Class)[1]', N'nchar(2)') as [Class],
        a.value(N'(./Style)[1]', N'nchar(2)') as [Style],
        a.value(N'(./ProductSubcategoryID)[1]', N'int') as [ProductSubcategoryID],
        a.value(N'(./ProductModelID)[1]', N'int') as [ProductModelID],
        a.value(N'(./SellStartDate)[1]', N'datetime') as [SellStartDate],
        a.value(N'(./SellEndDate)[1]', N'datetime') as [SellEndDate],
        a.value(N'(./DiscontinuedDate)[1]', N'datetime') as [DiscontinuedDate]
    from @messagebody.nodes('/AsyncSync/row') as r(a);
    
    -- Insert or Update the rows based on the incoming changes

    DELETE
    FROM #Production_Product
    WHERE [Action] = 'D'

    CREATE TABLE #ProdLevel (
		ProductID	int,
		StockLevel	int,
		Quantity	int
		)

	INSERT INTO #ProdLevel (
	    ProductID,
		StockLevel,
		Quantity
		)
	SELECT i.ProductID,
	    StockLevel = (i.[ReorderPoint] * 1.25),
		Quantity = (i.[ReorderPoint] * 1.25) - inv.Quantity
	FROM #Production_Product i
	INNER JOIN [Production].[Product] p  ON p.ProductID = i.ProductID
	INNER JOIN [Production].[ProductInventory] inv ON inv.ProductID = i.ProductID
	WHERE inv.Quantity < i.[ReorderPoint] * 1.25

	INSERT INTO [Production].[WorkOrder] (
	    ProductID,
		OrderQty,
		ScrappedQty,
		StartDate,
		DueDate,
		ModifiedDate
		)
	SELECT pl.ProductID,
	    OrderQty = pl.Quantity,
		ScrappedQty = 0,
		StartDate = GETDATE(),
		DueDate = DATEADD(d, 30, GETDATE()),
		ModifiedDate = GETDATE()
	FROM #ProdLevel pl

    UPDATE p
        SET p.[SafetyStockLevel] = pl.StockLevel
    FROM [Production].[Product] p
	INNER JOIN #Production_Product i ON p.ProductID = i.ProductID
	INNER JOIN #ProdLevel pl ON pl.ProductID = i.ProductID
    WHERE p.[SafetyStockLevel] <> pl.StockLevel;

-- Set updated ModifiedDate
	UPDATE p
    SET p.[ModifiedDate] = GETDATE()
    FROM [Production].[Product] p
    INNER JOIN #Production_Product i ON p.ProductID = i.ProductID;

  END TRY

  BEGIN CATCH
    -- Insert any errors into the ErrorLog table for later review and correction
    INSERT INTO dbo.ErrorLog (ErrorTime, UserName, ErrorNumber, ErrorSeverity,
    	ErrorState, ErrorProcedure, ErrorLine, ErrorMessage, MessageBody)
    VALUES (getdate(), USER_NAME(), ERROR_NUMBER(), ERROR_SEVERITY(),
    	ERROR_STATE(), ERROR_PROCEDURE(), ERROR_LINE(), ERROR_MESSAGE(), @messagebody)
  
  END CATCH

  RETURN

GO

IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[ProcessSyncData]') AND type in (N'P', N'PC'))
DROP PROCEDURE [dbo].[ProcessSyncData]
GO

CREATE PROCEDURE [dbo].[ProcessSyncData]
AS
SET NOCOUNT ON
	DECLARE @id INT;
	DECLARE @ch UNIQUEIDENTIFIER
	DECLARE @messagetypename NVARCHAR(256)
	DECLARE	@messagebody XML
	DECLARE @responsemessage XML;
	DECLARE @returnrows INT;

	WHILE (1=1)
	BEGIN
		BEGIN TRY

			SELECT TOP 1 @id=id, @ch=ch, @messagetypename=messagetypename, @messagebody=messagebody
			FROM [dbo].[BrokerMessages]
			ORDER BY id;
			SELECT @returnrows = @@ROWCOUNT
			
			IF (@returnrows > 0)
			BEGIN
				
				-- First extract the name of the table from the top row of the message
				DECLARE @TableName varchar(100);
				DECLARE @SyncTable TABLE (TableName varchar(100));

				INSERT INTO @SyncTable (TableName)
				select TOP 1 a.value(N'(./TableName)[1]', N'varchar(100)') as TableName
				from @messagebody.nodes('/AsyncSync/row') as r(a);

				SELECT @TableName = TableName FROM @SyncTable;

				-- Now call the procedure corresponding to the table passed into the message
				IF (@TableName = 'Production_Product')
					BEGIN
					EXEC dbo.ProcessProduction_Product @messagebody;
					END

				-- Delete the message so it's only processed once
				DELETE FROM [dbo].[BrokerMessages]
				WHERE id = @id;
			END
			ELSE
			BEGIN
				-- If we're out of messages, wait a minute and then check again
				WAITFOR DELAY '00:01:00';
			END

		END TRY
		BEGIN CATCH
				
			INSERT INTO dbo.ErrorLog (ErrorTime, UserName, ErrorNumber, ErrorSeverity,
				ErrorState, ErrorProcedure, ErrorLine, ErrorMessage, MessageBody)
			VALUES (getdate(), USER_NAME(), ERROR_NUMBER(), ERROR_SEVERITY(),
				ERROR_STATE(), ERROR_PROCEDURE(), ERROR_LINE(), ERROR_MESSAGE(), @messagebody)

		END CATCH
	END
GO

USE [AdventureWorks]
GO

IF OBJECT_ID ('dbo.tr_Production_Product_IUD','TR') IS NOT NULL
   DROP TRIGGER dbo.tr_Production_Product_IUD 
GO

CREATE TRIGGER [Production].tr_Production_Product_IUD
   ON  [Production].[Product]
   FOR INSERT, UPDATE, DELETE
AS 

  DECLARE @InitDlgHandle UNIQUEIDENTIFIER;
  DECLARE @ChangeMsg XML;
  DECLARE @ChangeCnt int;
  DECLARE @Action char(1);
  DECLARE @TableName varchar(100) = 'Production_Product';
  DECLARE @service_name nvarchar(512) = N'//AsyncSync/SyncService';
	
  SELECT @InitDlgHandle=[ch]
  FROM [dbo].[BrokerConversation]
  WHERE [service_name] = @service_name;
  
  BEGIN TRY
    IF USER_NAME() <> 'SyncUser'
    BEGIN
      
      -- If there are rows in both inserted and deleted, the change is an update
      SELECT @ChangeCnt = count(*)
        FROM .[Production].[Product] t
        INNER JOIN deleted d on t.[ProductID] = d.[ProductID]
        INNER JOIN inserted i on t.[ProductID] = i.[ProductID];
      if (@ChangeCnt > 0)
        BEGIN;
        SET @Action = 'U';
        END;
      ELSE
        BEGIN
        -- Since rows aren't in both inserted and deleted, then the change is an insert or a delete
        -- If there are rows in deleted, the change is a delete, and we only need the primary key value
        select @ChangeCnt = count(*) from deleted
        if (@ChangeCnt > 0)
          BEGIN
          SET @Action = 'D';
          END;
        
        -- If there are rows in inserted, the change is an insert and we need all the column values
        select @ChangeCnt = count(*) from inserted
        if (@ChangeCnt > 0)
          BEGIN
          SET @Action = 'I';
          END;
      
        END;

      if (@Action = 'D')
        BEGIN;
        -- Build the XML message, flagging it as a delete
        set @ChangeMsg = (
        SELECT @Action as [Action]
              ,@TableName as [TableName]
              ,t.[ProductID]
              ,t.[Name]
              ,t.[ProductNumber]
              ,t.[MakeFlag]
              ,t.[FinishedGoodsFlag]
              ,t.[Color]
              ,t.[SafetyStockLevel]
              ,t.[ReorderPoint]
              ,t.[StandardCost]
              ,t.[ListPrice]
              ,t.[Size]
              ,t.[SizeUnitMeasureCode]
              ,t.[WeightUnitMeasureCode]
              ,t.[Weight]
              ,t.[DaysToManufacture]
              ,t.[ProductLine]
              ,t.[Class]
              ,t.[Style]
              ,t.[ProductSubcategoryID]
              ,t.[ProductModelID]
              ,t.[SellStartDate]
              ,t.[SellEndDate]
              ,t.[DiscontinuedDate]
          FROM deleted t
          FOR XML RAW, ELEMENTS, ROOT ('AsyncSync')
          );
        END;
      ELSE
        BEGIN;
        -- Build the XML message, flagging it as an update
        set @ChangeMsg = (
        SELECT @Action as [Action]
              ,@TableName as [TableName]
              ,t.[ProductID]
              ,t.[Name]
              ,t.[ProductNumber]
              ,t.[MakeFlag]
              ,t.[FinishedGoodsFlag]
              ,t.[Color]
              ,t.[SafetyStockLevel]
              ,t.[ReorderPoint]
              ,t.[StandardCost]
              ,t.[ListPrice]
              ,t.[Size]
              ,t.[SizeUnitMeasureCode]
              ,t.[WeightUnitMeasureCode]
              ,t.[Weight]
              ,t.[DaysToManufacture]
              ,t.[ProductLine]
              ,t.[Class]
              ,t.[Style]
              ,t.[ProductSubcategoryID]
              ,t.[ProductModelID]
              ,t.[SellStartDate]
              ,t.[SellEndDate]
              ,t.[DiscontinuedDate]
          FROM [Production].[Product] p
          INNER JOIN inserted t on t.[ProductID] = p.[ProductID]
          FOR XML RAW, ELEMENTS, ROOT ('AsyncSync')
          );
        END;

    IF @ChangeMsg IS NOT NULL
      BEGIN
      BEGIN TRANSACTION;
      ;SEND ON CONVERSATION @InitDlgHandle
           MESSAGE TYPE [//AsyncSync/Product]
           (@ChangeMsg);
      --END CONVERSATION @InitDlgHandle;
      COMMIT TRANSACTION;
      END
    END
  END TRY
  BEGIN CATCH
  	IF XACT_STATE() <> 0
  		BEGIN
		-- Rollback any pending transaction
		ROLLBACK TRANSACTION;
		END	
	
	-- Insert the error information into the ErrorLog Production_Product for later reference
  	INSERT INTO dbo.ErrorLog (ErrorTime, UserName, ErrorNumber, ErrorSeverity,
  		ErrorState, ErrorProcedure, ErrorLine, ErrorMessage, messagebody)
  	VALUES (getdate(), USER_NAME(), ERROR_NUMBER(), ERROR_SEVERITY(),
  		ERROR_STATE(), ERROR_PROCEDURE(), ERROR_LINE(), ERROR_MESSAGE(), @ChangeMsg)
  
  END CATCH
  
GO

EXEC sp_settriggerorder @triggername=N'[Production].[tr_Production_Product_IUD]', @order=N'Last', @stmttype=N'DELETE'
GO

EXEC sp_settriggerorder @triggername=N'[Production].[tr_Production_Product_IUD]', @order=N'Last', @stmttype=N'INSERT'
GO

EXEC sp_settriggerorder @triggername=N'[Production].[tr_Production_Product_IUD]', @order=N'Last', @stmttype=N'UPDATE'
GO


ALTER QUEUE SyncProcessQueue
WITH ACTIVATION
(
	STATUS = ON,
	PROCEDURE_NAME = [dbo].[ProcessSyncMessages],
	MAX_QUEUE_READERS = 1,
	EXECUTE AS 'SyncUser'
)
GO
GRANT EXECUTE ON dbo.[ProcessSyncMessages] to SyncUser
GO
