CREATE TRIGGER [Production].[IUD_Product_StockLevel] ON [Production].[Product] 
 FOR INSERT, UPDATE, DELETE
 AS 
BEGIN
    SET NOCOUNT ON;

-- SafetyStockLevel must always be 125% of ReorderPoint value
-- Create WorkOrder records if new Stock Level increases
    CREATE TABLE #ProdLevel (
		ProductID	int,
		StockLevel	int,
		Quantity	int
		)

	INSERT INTO #ProdLevel (
	    ProductID,
		StockLevel,
		Quantity
		)
	SELECT i.ProductID,
	    StockLevel = (i.[ReorderPoint] * 1.25),
		Quantity = (i.[ReorderPoint] * 1.25) - inv.Quantity
	FROM inserted i
	INNER JOIN [Production].[Product] p  ON p.ProductID = i.ProductID
	INNER JOIN [Production].[ProductInventory] inv ON inv.ProductID = i.ProductID
	WHERE inv.Quantity < i.[ReorderPoint] * 1.25

	INSERT INTO [Production].[WorkOrder] (
	    ProductID,
		OrderQty,
		ScrappedQty,
		StartDate,
		DueDate,
		ModifiedDate
		)
	SELECT pl.ProductID,
	    OrderQty = pl.Quantity,
		ScrappedQty = 0,
		StartDate = GETDATE(),
		DueDate = DATEADD(d, 30, GETDATE()),
		ModifiedDate = GETDATE()
	FROM #ProdLevel pl

    UPDATE p
        SET p.[SafetyStockLevel] = pl.StockLevel
    FROM [Production].[Product] p
	INNER JOIN inserted i ON p.ProductID = i.ProductID
	INNER JOIN #ProdLevel pl ON pl.ProductID = i.ProductID
    WHERE p.[SafetyStockLevel] <> pl.StockLevel;

-- Set updated ModifiedDate
	UPDATE p
    SET p.[ModifiedDate] = GETDATE()
    FROM [Production].[Product] p INNER JOIN inserted i ON p.ProductID = i.ProductID;

END
GO
