USE [AdventureWorks]
GO

IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[ProcessProduction_Product]') AND type in (N'P', N'PC'))
DROP PROCEDURE [dbo].[ProcessProduction_Product]
GO

CREATE PROCEDURE [dbo].[ProcessProduction_Product]
	@messagebody XML
WITH EXECUTE AS 'SyncUser'
AS
  SET NOCOUNT ON
  BEGIN TRY
  
    -- Create a temporary table to hold the message results
    CREATE TABLE #Production_Product(
    	[Action] [nchar](1) NOT NULL,
    	[ProductID] [int] NOT NULL,
	[Name] [nvarchar](50) NULL,
	[ProductNumber] [nvarchar](25) NULL,
	[MakeFlag] [bit] NULL,
	[FinishedGoodsFlag] [bit] NULL,
	[Color] [nvarchar](15) NULL,
	[SafetyStockLevel] [smallint] NULL,
	[ReorderPoint] [smallint] NULL,
	[StandardCost] [money] NULL,
	[ListPrice] [money] NULL,
	[Size] [nvarchar](5) NULL,
	[SizeUnitMeasureCode] [nchar](3) NULL,
	[WeightUnitMeasureCode] [nchar](3) NULL,
	[Weight] [decimal](8, 2) NULL,
	[DaysToManufacture] [int] NULL,
	[ProductLine] [nchar](2) NULL,
	[Class] [nchar](2) NULL,
	[Style] [nchar](2) NULL,
	[ProductSubcategoryID] [int] NULL,
	[ProductModelID] [int] NULL,
	[SellStartDate] [datetime] NULL,
	[SellEndDate] [datetime] NULL,
	[DiscontinuedDate] [datetime] NULL
    );
    
    -- Use XQuery to shred the message and insert it into the temporary table
    INSERT INTO #Production_Product
    select  a.value(N'(./Action)[1]', N'nchar(1)') as [Action],
        a.value(N'(./ProductID)[1]', N'int') as [ProductID],
        a.value(N'(./Name)[1]', N'nvarchar(50)') as [Name],
        a.value(N'(./ProductNumber)[1]', N'nvarchar(25)') as [ProductNumber],
        a.value(N'(./MakeFlag)[1]', N'bit') as [MakeFlag],
        a.value(N'(./FinishedGoodsFlag)[1]', N'bit') as [FinishedGoodsFlag],
        a.value(N'(./Color)[1]', N'nvarchar(15)') as [Color],
        a.value(N'(./SafetyStockLevel)[1]', N'smallint') as [SafetyStockLevel],
        a.value(N'(./ReorderPoint)[1]', N'smallint') as [ReorderPoint],
        a.value(N'(./StandardCost)[1]', N'money') as [StandardCost],
        a.value(N'(./ListPrice)[1]', N'money') as [ListPrice],
        a.value(N'(./Size)[1]', N'nvarchar(5)') as [Size],
        a.value(N'(./SizeUnitMeasureCode)[1]', N'nchar(3)') as [SizeUnitMeasureCode],
        a.value(N'(./WeightUnitMeasureCode)[1]', N'nchar(3)') as [WeightUnitMeasureCode],
        a.value(N'(./Weight)[1]', N'decimal(8, 2)') as [Weight],
        a.value(N'(./DaysToManufacture)[1]', N'int') as [DaysToManufacture],
        a.value(N'(./ProductLine)[1]', N'nchar(2)') as [ProductLine],
        a.value(N'(./Class)[1]', N'nchar(2)') as [Class],
        a.value(N'(./Style)[1]', N'nchar(2)') as [Style],
        a.value(N'(./ProductSubcategoryID)[1]', N'int') as [ProductSubcategoryID],
        a.value(N'(./ProductModelID)[1]', N'int') as [ProductModelID],
        a.value(N'(./SellStartDate)[1]', N'datetime') as [SellStartDate],
        a.value(N'(./SellEndDate)[1]', N'datetime') as [SellEndDate],
        a.value(N'(./DiscontinuedDate)[1]', N'datetime') as [DiscontinuedDate]
    from @messagebody.nodes('/SBETL/row') as r(a);
    
    -- Insert or Update the rows based on the incoming changes

    DELETE
    FROM #Production_Product
    WHERE [Action] = 'D'

    CREATE TABLE #ProdLevel (
		ProductID	int,
		StockLevel	int,
		Quantity	int
		)

	INSERT INTO #ProdLevel (
	    ProductID,
		StockLevel,
		Quantity
		)
	SELECT i.ProductID,
	    StockLevel = (i.[ReorderPoint] * 1.25),
		Quantity = (i.[ReorderPoint] * 1.25) - inv.Quantity
	FROM #Production_Product i
	INNER JOIN [Production].[Product] p  ON p.ProductID = i.ProductID
	INNER JOIN [Production].[ProductInventory] inv ON inv.ProductID = i.ProductID
	WHERE inv.Quantity < i.[ReorderPoint] * 1.25

	INSERT INTO [Production].[WorkOrder] (
	    ProductID,
		OrderQty,
		DueDate,
		ModifiedDate
		)
	SELECT pl.ProductID,
	    OrderQty = pl.Quantity,
		DueDate = DATEADD(d, 30, GETDATE()),
		ModifiedDate = GETDATE()
	FROM #ProdLevel pl

    UPDATE p
        SET p.[SafetyStockLevel] = pl.StockLevel
    FROM [Production].[Product] p
	INNER JOIN #Production_Product i ON p.ProductID = i.ProductID
	INNER JOIN #ProdLevel pl ON pl.ProductID = i.ProductID
    WHERE p.[SafetyStockLevel] <> pl.StockLevel;

-- Set updated ModifiedDate
	UPDATE p
    SET p.[ModifiedDate] = GETDATE()
    FROM [Production].[Product] p
    INNER JOIN #Production_Product i ON p.ProductID = i.ProductID;

  END TRY

  BEGIN CATCH
    -- Insert any errors into the ErrorLog table for later review and correction
    INSERT INTO dbo.ErrorLog (ErrorTime, UserName, ErrorNumber, ErrorSeverity,
    	ErrorState, ErrorProcedure, ErrorLine, ErrorMessage, MessageBody)
    VALUES (getdate(), USER_NAME(), ERROR_NUMBER(), ERROR_SEVERITY(),
    	ERROR_STATE(), ERROR_PROCEDURE(), ERROR_LINE(), ERROR_MESSAGE(), @messagebody)
  
  END CATCH

  RETURN

GO
